from pathlib import Path

import revelation

REVEAL_URL = "https://github.com/hakimel/reveal.js/archive/master.zip"
MATHJAX_URL = "https://github.com/mathjax/MathJax/archive/2.7.9.zip"

STATIC_ROOT = Path(revelation.__file__).parent / "static"

REVEALJS_DIR = STATIC_ROOT / "revealjs"
MATHJAX_DIR = STATIC_ROOT / "mathjax"



"""
Main revelation module

It has the Revelation main class that creates the webserver do run
the presentation
"""

import re
from pathlib import Path
from typing import Optional

from jinja2 import Environment, PackageLoader, select_autoescape
from werkzeug.middleware.shared_data import SharedDataMiddleware
from werkzeug.wrappers import Request, Response

from .config import Config
from .utils import normalize_newlines


class Revelation(object):
    """
    Main revelation app class that instantiates the server and handles
    the requests
    """

    presentation: Path
    config: Config
    media: Optional[Path]
    theme: Optional[Path]
    style: Optional[Path]
    static: Path = Path(__file__).parent.resolve() / "static"

    def __init__(
        self,
        presentation: Path,
        config: Optional[Path] = None,
        media: Optional[Path] = None,
        theme: Optional[Path] = None,
        style: Optional[Path] = None,
    ):
        """
        Initializes the server and creates the environment for the presentation
        """
        self.presentation = presentation
        self.config = Config(config)
        self.media = media
        self.theme = theme
        self.style = style

        shared_data = dict()
        shared_data.update(self.parse_shared_data(self.static))
        shared_data.update(self.parse_shared_data(self.media))
        shared_data.update(self.parse_shared_data(self.theme))
        shared_data.update(self.parse_shared_data(self.style))

        self.wsgi_app = SharedDataMiddleware(self._wsgi_app, shared_data)

    def parse_shared_data(self, shared_root: Optional[Path]) -> dict:
        """
        Parse aditional shared_data if it exists
        """
        if shared_root and shared_root.exists():
            shared_root = shared_root.resolve()
            shared_url = f"/{shared_root.name}"

            return {shared_url: str(shared_root)}

        return {}

    def load_slides(
        self,
        path: Path,
        section_separator: str,
        vertical_separator: str,
    ):
        """
        Get slides file from the given path, loads it and split into list
        of slides.

        :return: a list of strings with the slides content
        """
        with path.open("rb") as fp:
            slides = normalize_newlines(fp.read().decode("utf-8"))

        return [
            re.split(f"^{vertical_separator}$", section, flags=re.MULTILINE)
            for section in re.split(
                f"^{section_separator}$", slides, flags=re.MULTILINE
            )
        ]

    def get_theme(self, theme_name: str):
        reveal_theme = f"static/revealjs/dist/theme/{theme_name}.css"
        fullpath_theme = Path(__file__).parent.resolve() / reveal_theme

        if fullpath_theme.is_file():
            return reveal_theme

        return theme_name

    def dispatch_request(self, _: Optional[Request] = None):
        env = Environment(
            loader=PackageLoader("revelation", "templates"),
            autoescape=select_autoescape(["html"]),
        )

        context = {
            "meta": self.config.get("REVEAL_META"),
            "slides": self.load_slides(
                self.presentation,
                str(self.config.get("REVEAL_SLIDE_SEPARATOR")),
                str(self.config.get("REVEAL_VERTICAL_SLIDE_SEPARATOR")),
            ),
            "config": self.config.get("REVEAL_CONFIG"),
            "theme": self.get_theme(str(self.config.get("REVEAL_THEME"))),
            "style": getattr(self.style, "name", None),
        }

        template = env.get_template("presentation.html")

        return Response(
            template.render(**context), headers={"content-type": "text/html"}
        )

    def _wsgi_app(self, environ, start_response):
        request = Request(environ)
        response = self.dispatch_request(request)

        return response(environ, start_response)

    def __call__(self, environ, start_response):
        return self.wsgi_app(environ, start_response)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['revelation']

package_data = \
{'': ['*'], 'revelation': ['templates/*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0',
 'Werkzeug[watchdog]>=2.0.1,<3.0.0',
 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['revelation = revelation.cli:cli',
                     'rv = revelation.cli:cli']}

setup_kwargs = {
    'name': 'revelation',
    'version': '2.0.0',
    'description': 'Make awesome reveal.js presentations with revelation',
    'long_description': "# Revelation\n\n[![PyPI](https://img.shields.io/pypi/v/revelation.svg)](https://pypi.org/project/revelation/)\n[![PyPI - License](https://img.shields.io/pypi/l/revelation.svg)](https://pypi.org/project/revelation/)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/revelation.svg)](https://pypi.org/project/revelation/)\n[![Actions Status](https://github.com/humrochagf/revelation/workflows/CI/badge.svg)](https://github.com/humrochagf/revelation/actions)\n[![Coverage Status](https://coveralls.io/repos/github/humrochagf/revelation/badge.svg?branch=main)](https://coveralls.io/github/humrochagf/revelation?branch=main)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n\n[revelation](https://github.com/humrochagf/revelation) is a cli tool that makes your [revealjs](https://github.com/hakimel/reveal.js) presentations nice and easy using markdown and serving it locally.\n\n## Features\n\n- Presentation server to run it locally when you are offline.\n- Presentation creation from template with a single command.\n- Custom theming support with css.\n- Export to static html tool.\n- Debug mode with server auto refresh.\n\n## Installation\n\nYou can install it with:\n\n```shell\npip install revelation\n```\n\nOr with [pipx](https://pypa.github.io/pipx/):\n\n```shell\npipx install revelation\n```\n\n## Usage\n\n### Install/Update revealjs files\n\nRevelation depends on revealjs to run its presentation and every command bellow will check and install it if revealjs isn't installed yet.\n\nBut, you can also manually install or even update it to a different version with the `installreveal` command:\n\n```shell\nrevelation installreveal\n```\n\n### Creating a new Presentation\n\nTo create a new presentation you can use `mkpresentation` command that will setup a new presentation using the base layout for you:\n\n```shell\nrevelation mkpresentation mypresentation\n```\n\n### Running the Presentation\n\nYou can start presenting with the `start` command on your presentation markdown file:\n\n```shell\ncd mypresentation\nrevelation start slides.md\n```\n\n### Static Export\n\nTo export the presentation as static HTML content use the command:\n\n```shell\nrevelation mkstatic slides.md\n```\n\n### PDF Export\n\nPresentations can be exported to PDF via a special print stylesheet. This feature will be described using [Google Chrome](https://google.com/chrome) or [Chromium](https://www.chromium.org/Home), but I got the same results using [Firefox](https://www.mozilla.org/en-US/firefox/new/).\n\n1. Run the presentation with revelation.\n2. Open your browser with the `print-pdf` as query string like : `localhost:5000/?print-pdf`.\n3. Open the in-browser print dialog (CTRL+P or CMD+P).\n4. Change the **Destination** setting to **Save as PDF**.\n5. Change the **Layout** to **Landscape**.\n6. Change the **Margins** to **None**.\n7. Enable the **Background graphics** option.\n8. Click **Save**.\n\nAlternatively you can use the [decktape](https://github.com/astefanutti/decktape) project.\n\n## Presentation Setup\n\nThe base presentation file structure looks like this:\n\n```\npresentation/\n|- media/\n|- theme/\n|- config.py\n|- slides.md\n```\n\n### The slides.md File\n\nThis is your presentation file written using markdown with some especial tags described on [markdown section](#markdown) and is placed on your presentation root folder.\n\nSplit your slides by setting up a *slide separator* and *vertical slide separator* into **REVEAL_CONFIG**. Default separator are `---` and `---~`.\n\n### The media folder\n\nBy default, revelation looks for a folder called **media** inside your presentation root folder. All media placed inside it can be referenced on your presentation by the path `/media`:\n\n```md\n![Python Logo](media/python.png)\n```\n\nYou can define a custom media path using the `--media` option on revelation `start` command.\n\n### The theme folder\n\nYou can create your custom theme file and place it inside a **theme** folder and reference it at the configuration file by the option `--theme`.\n\n### The config.py File\n\nThe configuration file located in the root folder of your presentation, allows you to customize your presentation. This file is optional and have the following configuration options:\n\n- **REVEAL_META**: Metadata of your presentation, mostly to identify the author and title.\n- **REVEAL_THEME**: A string where you can select the theme you would like to use. All revealjs base themes are available.\n- **REVEAL_CONFIG**: A python dictionary with the [revealjs configuration attributes](https://revealjs.com/config/) but using python types (e.g.: true is python boolean True)\n\nOnce you create a new presentation, all configuration values will be there for you to customize.\n\n## Markdown\n\nThe markdown used on the presentation files support everything that [revealjs docs](https://revealjs.com/markdown/) allows to place inside the `data-template` section.\n",
    'author': 'Humberto Rocha',
    'author_email': 'humrochagf@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
